package com.ibm.websphere.fabric.insurance.pc.contextextractor;

import java.util.logging.Logger;

import com.ibm.websphere.fabric.da.PendingRequest;
import com.ibm.websphere.fabric.da.context.Context;
import com.ibm.websphere.fabric.da.plugin.ContextExtractor;
import com.ibm.websphere.fabric.da.plugin.UnexpectedContentException;
import com.ibm.websphere.fabric.da.types.TypedValue;
import com.ibm.websphere.fabric.insurance.pc.service.util.DataAssigmentHelper;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

public class LineOfBusinessAssertionContextExtractorImpl implements
		ContextExtractor {
    /**
     * <code>logger</code> is used for loggin info.
     */
    private final Logger logger = Logger.getLogger("LineOfBusinessAssertionContextExtractorImpl");

	
	
    /**
     * URI for state based assertion.
     */
    private static final String LINE_OF_BUSINESS_BASED_ASSERTION = "http://www.ibm.com/websphere/fabric/insurance/acord/1.10.0/pc/assertions/schema#hasLineOfBusiness";
    
	private static final String FABRIC_GOVERNANCE = "fc://fabric/governance/fabricproject-inst#3c765894-632c-4866-810a-ac74e16b16b0";

	private static final String FABRIC_GOVERNANCE_VALUE = "http://www.webifysolutions.com/2005/10/catalog/governance#FabricProject";

	/**
	 * Default constructor.
	 */
	public LineOfBusinessAssertionContextExtractorImpl() {
		super();
	}

	/**
	 * Return a reference to the component service instance for this implementation
	 * class.  This method should be used when passing this service to a partner reference
	 * or if you want to invoke this component service asynchronously.    
	 *
	 * @generated (com.ibm.wbit.java)
	 */
	@SuppressWarnings("unused")
	private ContextExtractor getMyService() {
		return (ContextExtractor) ServiceManager.INSTANCE.locateService("self");
	}

	/* (non-Javadoc)
	 * @see com.ibm.websphere.fabric.da.plugin.ContextExtractor#extractContext(PendingRequest arg0) 
	 */
	public Context extractContext(PendingRequest pendingRequest)throws UnexpectedContentException {
		
		
        DataObject requestDataObject = (DataObject) pendingRequest.getFirstBodyElement();
        logger.info("requestDataObject " + requestDataObject);

        String xpath = "//LOB";
        String lob = (String) DataAssigmentHelper.getValueFromXPath(requestDataObject, xpath);
        logger.info("lob:" + lob);
		
        Context context = pendingRequest.getContext();
        logger.info("context.:" + context.getSelectionPropertyNames());

        if (lob != null) {
            TypedValue assersionValue = new TypedValue(lob);
            
            if (lob.equalsIgnoreCase("Umbrella")) {
            
            	assersionValue = new TypedValue("HOME");
            }
            context.setSelectionProperty(LINE_OF_BUSINESS_BASED_ASSERTION, assersionValue);
        }
       
        logger.info("line of business based assertion set to context.");
        
		TypedValue governanceFabricProjectValue = new TypedValue(FABRIC_GOVERNANCE);

		context.setSelectionProperty(FABRIC_GOVERNANCE_VALUE,governanceFabricProjectValue);

		
        return context;

}

}