/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 *
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */
package com.ibm.websphere.fabric.insurance.pc.contextextractor;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import com.ibm.websphere.fabric.da.PendingRequest;
import com.ibm.websphere.fabric.da.context.Context;
import com.ibm.websphere.fabric.da.plugin.ContextExtractor;
import com.ibm.websphere.fabric.da.plugin.UnexpectedContentException;
import com.ibm.websphere.fabric.da.types.TypedValue;
import com.ibm.websphere.fabric.insurance.pc.service.util.BpelContextHandler;
import com.ibm.websphere.fabric.insurance.pc.service.util.DataAssigmentHelper;
import com.ibm.websphere.fabric.insurance.pc.service.util.IConstants;
import com.ibm.ws.fabric.da.sca.util.SdoUtil;
import commonj.sdo.DataObject;

/**
 * Sets the role based assertion for the current interface.
 * 
 * @author govindm
 */
public class RoleAssertionContextExtractorImpl implements ContextExtractor {
    
    /**
     * <code>logger</code> is used to log info.
     */
    private final Logger logger = Logger.getLogger("RoleAssertionContextExtractorImpl");

    /**
     * URI for role based assertion.
     */
    private static final String ROLE_BASED_ASSERTION = 
        "http://w3.ibm.com/contentpack/insurance/pc/refimpl/assertions#has_role";
    
	private static final String FABRIC_GOVERNANCE = "fc://fabric/governance/fabricproject-inst#3c765894-632c-4866-810a-ac74e16b16b0";

	private static final String FABRIC_GOVERNANCE_VALUE = "http://www.webifysolutions.com/2005/10/catalog/governance#FabricProject";


    /**
     * Default constructor.
     */
    public RoleAssertionContextExtractorImpl() {

        super();
    }


    /**
     * @see com.ibm.websphere.fabric.da.plugin.ContextExtractor#
     *      extractContext(com.ibm.websphere.fabric.da.PendingRequest)
     */
    public Context extractContext(PendingRequest pendingRequest) throws UnexpectedContentException {

        Context context = pendingRequest.getContext();
        logger.info("context.:" + context.getSelectionPropertyNames());

        
        DataObject requestDataObject = (DataObject) pendingRequest.getFirstBodyElement();
        //logger.info("requestDataObject " + SdoUtil.printTree(requestDataObject));

        String xpath = "//ClaimsSvcRq[1]/ClaimStatusInqRq[1]/ClaimStatusInqInfo/InsuredOrPrincipal/" +
        "GeneralPartyInfo/Addr[1]/StateProvCd";
        String stateCode = (String) DataAssigmentHelper.getValueFromXPath(requestDataObject, xpath);
        logger.info("State Code:" + stateCode);
        
        if (!(stateCode.equals("TX") || stateCode.equals("TN"))) {
            stateCode = IConstants.OTHER;
        }

        // Save state value
        Map<Object, Object> map = BpelContextHandler.getContextProperties();
        if (map == null) {
            map = new HashMap<Object, Object>();
        }
        map.put(IConstants.STATE, stateCode);
        BpelContextHandler.setContextProperties(map);
        
        xpath = "//SignonRq/SignonPswd/SignonRoleCd";
        String inputSignonRoleCd = (String) DataAssigmentHelper.getValueFromXPath(requestDataObject, xpath);
        logger.info("Got SignonRoleCd: " + inputSignonRoleCd);

        TypedValue assersionValue = new TypedValue(inputSignonRoleCd);
        context.setSelectionProperty(ROLE_BASED_ASSERTION, assersionValue);

        logger.info("Role Based assertion set to context.");
        
        logger.info("context.ContextIdentifier:" + context.getContextIdentifier());
        Set<String> set = context.getSelectionPropertyNames();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
			String prop = (String) iterator.next();
			logger.info("context.property:" + prop);
			logger.info("context.value:" + context.getSelectionProperty(prop));			
		}
        
    	TypedValue governanceFabricProjectValue = new TypedValue(
				FABRIC_GOVERNANCE);

		context.setSelectionProperty(FABRIC_GOVERNANCE_VALUE,
				governanceFabricProjectValue);
        return context;
    }

}