/***********************************************
 *
 * 5724-R31, 5655-S30
 *
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.fabric.insurance.pc.service.util;

import java.io.InputStream;
import java.math.BigDecimal;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.ibm.websphere.fabric.da.util.SdoXPathUtil;
import com.ibm.websphere.fabric.da.util.XPathSearchResult;
import com.ibm.websphere.fabric.industry.support.util.IndustrySupportUtil;
import com.ibm.ws.fabric.da.sca.util.SdoUtil;
import commonj.sdo.DataObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Data assignments done in this util class.
 *
 * @author govindm,harjinderm
 */
public final class AcordDataAssigmentHelper {

	static String ACORD_NS = "http://www.ACORD.org/standards/PC_Surety/ACORD1.10.0/xml/";
	
	private static final Log LOG = LogFactory.getLog(AcordDataAssigmentHelper.class);
	//static Map<String, String> claimStatusMap = new HashMap<String, String>(); 
	
	//static Map<String, String> lobMap = new HashMap<String, String>(); 

    private AcordDataAssigmentHelper() {
//    	claimStatusMap.put("C", "Closed");
//    	claimStatusMap.put("S", "Open");
//    	claimStatusMap.put("R", "Rejected");
//    	lobMap.put("A", "AUTO");
//    	lobMap.put("H", "HOME");
//    	lobMap.put("U", "Umbrella");
    }

    /**
     * Required details are set in the custDetailsResponse with the values from acordResponse.
     *
     * @param acordRequest
     * @param ClaimInquiryResquest
     */
    public static void assignInputToAordRequest(DataObject acordRequest, DataObject claimInquiryRequest) {

    			DataObject signonRq = acordRequest.createDataObject("SignonRq");

    			DataObject signonPswd = signonRq.createDataObject("SignonPswd");
    			signonPswd.setString("SignonRoleCd", "Agent");

    			DataObject custId = signonPswd.createDataObject("CustId");

    			custId.setString("CustPermId", "ptaylor");
    			custId.setString("CustLoginId", "correct");
    			
    			//signonPswd.createDataObject("CustPswd").setString("Pswd", "String");
    			DataObject custPswd = signonPswd.createDataObject("CustPswd");
    			custPswd.setString("Pswd", "String");
    			custPswd.setString("EncryptionTypeCd", "NONE");
    			
    			DataObject clientDt = signonRq.createDataObject("ClientDt");
    			DataObject custLangPref = signonRq.createDataObject("CustLangPref");
    			DataObject clientApp = signonRq.createDataObject("ClientApp");
    			custPswd.setString("Org", "Applied Systems");
    			custPswd.setString("Name", "WinTam");
    			    			

    			DataObject generalPartyInfoDO = acordRequest.createDataObject("ClaimsSvcRq")
                .createDataObject("ClaimStatusInqRq").createDataObject("ClaimStatusInqInfo")
                .createDataObject("InsuredOrPrincipal").createDataObject("GeneralPartyInfo");
    			
                 DataObject personNameDO = generalPartyInfoDO.createDataObject("NameInfo")
                .createDataObject("PersonName");
            	personNameDO.createDataObject("GivenName").set("value", (claimInquiryRequest.getString("FirstName")).toLowerCase());
            	personNameDO.createDataObject("Surname").set("value", (claimInquiryRequest.getString("LastName")).toLowerCase());

            	DataObject addrDO = generalPartyInfoDO.createDataObject("Addr");
            	addrDO.createDataObject("StateProvCd").set("value", "TX");
            	addrDO.createDataObject("PostalCode").set("value", claimInquiryRequest.getString("Zip"));

    }

    public static Object getValueFromXPath(DataObject objectDO, String xpath) {
        Object value = null;
        XPathSearchResult xpsresult = SdoXPathUtil.getViaXpath(objectDO, xpath);
        if(xpsresult!= null) {
	        if (xpsresult.isDataObject()) {
                value = ((DataObject)xpsresult.getValue()).get("value");
            } else {
                value = xpsresult.getValue();
            }
	    }
        return value;
    }

    public static void createAcordResp(DataObject claimInquiryResponse) {

    	claimInquiryResponse = IndustrySupportUtil.createDO("http://www.ibm.com/xmlns/prod/websphere/icp/2009/12/insurance/schema/bestpractices/schema/ClaimsInquiry","ClaimInquiry_Response");
    	claimInquiryResponse.setString("InsuredName", "james");
    	claimInquiryResponse.setString("PolicyNumber", "2002/002202");
    	claimInquiryResponse.setString("ClaimNumber", "11");
    	claimInquiryResponse.setString("ClaimReportDate", "9/24/2008");
    	claimInquiryResponse.setString("LOB", "Auto");
    	claimInquiryResponse.setString("ClaimStatus", "C");
    	claimInquiryResponse.setString("ClaimLifeCycle", "Test");
    	claimInquiryResponse.setString("Fname", "james");
    	claimInquiryResponse.setString("Lname", "smith");
    	claimInquiryResponse.setString("Address1", "add1-");
    	claimInquiryResponse.setString("Address2", "add2-");
    	claimInquiryResponse.setString("City", "city1");
    	claimInquiryResponse.setString("StateProvCd", "TX");
    	claimInquiryResponse.setString("PostalCode", "98789");
    	claimInquiryResponse.setString("AccountNumberId", "1234");
    	claimInquiryResponse.setString("ClaimStatusDt", "5/10/2008");
    	claimInquiryResponse.setString("IncidentDesc", "Test");
    	claimInquiryResponse.setString("PaymentTypeCd", "VISA");
    	claimInquiryResponse.setString("TotalPaymentAmt", "1230");
    	claimInquiryResponse.setString("MethodPaymentCd", "CASH");
    	claimInquiryResponse.setString("PaymentDt", "5/10/2008");

    }


    public static void assignAordResponseToOutput(DataObject acordResponse, DataObject claimInquiryResponse) {

    	SimpleDateFormat df = new SimpleDateFormat("dd-MM-yyyy");
    	
    	String firstNameInRes = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/NameInfo[1]/PersonName/GivenName");
    	
    	if (firstNameInRes==null || "".equals(firstNameInRes)) {

    		
    		XPathSearchResult xpsresult = SdoXPathUtil.getViaXpath(acordResponse, "//MsgStatusDesc");
    		if (xpsresult != null ) {

	    		DataObject msgStatusDescBO = (DataObject) xpsresult.getValue();

	    		
	    		String error_msg = (String)msgStatusDescBO.get("value");
	    		
	    		claimInquiryResponse.setString("ErrorMsg",error_msg);
	    		claimInquiryResponse.setString("claimState", "Close");
    		}
    	} else {
			String lastNameInRes = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/NameInfo[1]/PersonName/Surname");
	    	String policyNoInRes = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/PolicyNumber");
			String addr1 = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/Addr[1]/Addr1");
			String addr2 = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/Addr[1]/Addr2");
			String city = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/Addr[1]/City");
			String stateProvCd = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/Addr[1]/StateProvCd");
			String postalCode = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/InsuredOrPrincipal/GeneralPartyInfo/Addr[1]/PostalCode");
	
	    	String accountNumberId = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo/AccountNumberId");
	
	    	String claimStatusCd = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimStatusCd");
	    	Object obj = AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
			"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimStatusDt");
	    	
	    	Date claimStatusDt = null;
	    	if (obj != null) {    		
	    		claimStatusDt = (Date) obj;
	    		//claimInquiryResponse.setString("ClaimStatusDt", dateFormat.format(claimStatusDt));
	    		claimInquiryResponse.setDate("ClaimStatusDt", claimStatusDt);
	    	}
	    	
	    	obj = AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ReportedDt");
	    	Date reportedDt = null;
	    	if (obj != null) {
	    		reportedDt = (Date) obj; 
	    		//claimInquiryResponse.setString("ClaimReportDate", dateFormat.format(reportedDt));
	    		claimInquiryResponse.setDate("ClaimReportDate", reportedDt);
	    	}    		
	    	
	    	obj = AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/LossDt");
	    	Date lossDt = null;
	    	if (obj != null) {
	    		lossDt = (Date) obj; 
	    		//claimInquiryResponse.setString("lossDt", dateFormat.format(lossDt));
	    		claimInquiryResponse.setDate("lossDt", lossDt);
	    	}
	    	
	    	String incidentDesc = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/IncidentDesc");
	
	    	String paymentTypeCd = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsPayment[1]/PaymentTypeCd");
	    	BigDecimal totalPaymentAmt = (BigDecimal) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimsPayment[1]/TotalPaymentAmt/Amt");
	    	String methodPaymentCd = (String) AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
	    		"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimsPayment[1]/MethodPaymentCd");
	    	
	    	obj = AcordDataAssigmentHelper.getValueFromXPath(acordResponse,
				"//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimsPayment[1]/PaymentDt");
	    	Date paymentDt = null;
	    	if (obj != null) {
	    		paymentDt = (Date) obj; 
	    		//claimInquiryResponse.setString("PaymentDt", dateFormat.format(paymentDt));
	    		claimInquiryResponse.setDate("PaymentDt", paymentDt);
	    	}
	
	  //  	String claimStatus = claimStatusMap.get(claimStatusCd.charAt(0));
	    	String claimStatus = "";
	    	switch(claimStatusCd.charAt(0)){
	    		case 'C':
	    			claimStatus = "Closed";
	    			break;
	    		case 'S':
	    			claimStatus = "Open";
	    			break;
	    		case 'R':
	    			claimStatus = "Rejected";
	    			break;
	    	}
	    	
	    	if (lastNameInRes.length()> 0) {
	    		lastNameInRes = Character.toUpperCase(lastNameInRes.charAt(0)) +  lastNameInRes.substring(1);
	    	}
	    	 
	    	ClaimDetailInfo claimDetailInfo =DataReader.getInstance().getClaimDetailInfoByFirstName(firstNameInRes);
	    	String lob = claimDetailInfo.getLOB();
	    	String address1 = claimDetailInfo.getAddress1();
	    	String address2 = claimDetailInfo.getAddress2();
	    	String _city = claimDetailInfo.getCity();
	    	//String lob = lobMap.get(claimDetailInfo_lob.charAt(0));
	    	switch(lob.charAt(0)){
			case 'A':
				lob = "AUTO";
				break;
			case 'H':
				lob = "HOME";
				break;
			case 'U':
				lob = "Umbrella";
				break;
	    	}
		
	    	String claimNo =claimDetailInfo.getclaimNo();
	    	claimInquiryResponse.setString("InsuredName", 
	    			Character.toUpperCase(firstNameInRes.charAt(0)) +  firstNameInRes.substring(1)+ " " + lastNameInRes);
	    	claimInquiryResponse.setString("PolicyNumber", policyNoInRes);
	    	claimInquiryResponse.setString("ClaimNumber", claimNo);    	
	    	claimInquiryResponse.setString("LOB", lob);
	    	String emailID = firstNameInRes+"@email.com";
	    	String homePhoneNo = claimDetailInfo.getHomephone();
	    	String dayPhoneNo = claimDetailInfo.getDayphone();
	    	claimInquiryResponse.setString("ClaimStatus", claimStatus);
	    	claimInquiryResponse.setString("Fname", firstNameInRes);
	    	claimInquiryResponse.setString("Lname", lastNameInRes);
	    	claimInquiryResponse.setString("Address1", address1);
	    	claimInquiryResponse.setString("Address2", address2);
	    	claimInquiryResponse.setString("City", _city);
	    	claimInquiryResponse.setString("StateProvCd", stateProvCd);
	    	claimInquiryResponse.setString("PostalCode", postalCode);
	    	claimInquiryResponse.setString("AccountNumberId", accountNumberId);
	    	claimInquiryResponse.setDate("ClaimStatusDt", claimStatusDt);
	    	claimInquiryResponse.setString("IncidentDesc", incidentDesc);
	    	claimInquiryResponse.setString("PaymentTypeCd", paymentTypeCd);
	    	claimInquiryResponse.setString("TotalPaymentAmt", totalPaymentAmt.toString());
	    	claimInquiryResponse.setString("MethodPaymentCd", methodPaymentCd);
	    	claimInquiryResponse.setDate("PaymentDt", paymentDt);
	  
	    	Calendar cal = Calendar.getInstance(); 
	    	cal.setTime(reportedDt);
	    	// Add 3 days from the calendar
	    	cal.add(Calendar.DATE, +3);
	    	Date adjudicationDt = cal.getTime();  	
	    	claimInquiryResponse.setDate("AdjudicationDt", adjudicationDt);
	    	
	    	cal.setTime(reportedDt);
	    	cal.add(Calendar.DATE, -55);
	    	Date effectiveDt = cal.getTime();
	    	claimInquiryResponse.setDate("EffectiveDt", effectiveDt);
	    	
	    	cal.setTime(reportedDt);
	    	cal.add(Calendar.DATE, +124);
	    	Date expirationDt = cal.getTime();
	    	claimInquiryResponse.setDate("ExpirationDt", expirationDt);
	    	
	    	
	    	cal.setTime(reportedDt);
	    	cal.add(Calendar.DATE, +1);
	    	Date inspectionDt = cal.getTime();
	    	Format formatter = new SimpleDateFormat("yyyy-MM-dd");
	    	String inspectionDate = formatter.format(inspectionDt);
	    	claimInquiryResponse.set("InspectionDt", inspectionDate);
	    	//ateFormat.getDateInstance(DateFormat.SHORT).format(inspectionDt);
	    	//claimInquiryResponse.setString("AdjudicatorName", "Michael Stafford 1-800-100-2000, Extn - 1234");
	    	
	    	claimInquiryResponse.setDate("lossDt", lossDt);
	    	claimInquiryResponse.setString("emailId", emailID);
	    	claimInquiryResponse.setString("claimState", "Close");
	    	claimInquiryResponse.setString("ContactNumber", homePhoneNo);

  
    	}   	
    }
    
//    public static InputStream getAcordRequestReader() {
//    	InputStream is = AcordDataAssigmentHelper.class.getResourceAsStream("acord_request_ri.xml");
//    	debug("is===----="+is);
//    	return is;
//    }

    public static DataObject createACORDRequest() {
    	DataObject acord = null;
    
        try{
        	InputStream is = AcordDataAssigmentHelper.class.getResourceAsStream("acord_request_ri.xml");
        	acord = com.ibm.websphere.fabric.industry.support.util.SDOTransformationUtils.createDataObject(is);
	    }
	    catch(Exception e){
	    	
	    	e.printStackTrace();
	    }	
	    return acord;
	}

    public static ClaimDetailInfo getCsvDetails(DataObject claimInquiryRequest){
    	String claimNumber = "";
    	ClaimDetailInfo  claimDetailInfo = null ;
    	try{
    		claimNumber = claimInquiryRequest.getString("ClaimNumber");
        	claimDetailInfo = DataReader.getInstance().getClaimDetailInfoByClaimNO(claimNumber);
    		
    	}catch (Exception e) {
    		LOG.info("Invalid Claim Number");
            LOG.error(e.getMessage(), e);
		}
    	
    	return claimDetailInfo;
    }
	public static DataObject getContext() {

    	DataObject context = IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextTypeExt");
    	//DataObject contextProperty_01 = IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		DataObject contextProperty_02 = IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		DataObject contextProperty_03 = IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");
		DataObject contextProperty_04 = IndustrySupportUtil.createDO("http://www.webifysolutions.com/2006/01/prism-context","ContextPropertyType");

		//DataObject contextId = IndustrySupportUtil.createDO("http://docs.oasis-open.org/wscaf/2005/10/wsctx","contextIdentifierType");
		//contextId.set("value", "ID:ISL106-1165-1240916946140-0:1");
		//contextProperty_01.setString("name","http://www.webifysolutions.com/2005/10/catalog/governance#Environment");
		//contextProperty_01.setString("value","fc://fabric/governance/default-inst#default");
		contextProperty_02.setString("name","http://www.webifysolutions.com/context/debug");
		contextProperty_02.setString("value","true");
		contextProperty_03.setString("name","http://www.webifysolutions.com/context/interface");
		contextProperty_03.setString("value","http://ClaimSummaryAcordGtwy/ClaimSummaryAcordGtwyBP/ClaimSummaryAcordGtwyBPInterface#ClaimSummaryAcordGtwyBP");
		contextProperty_04.setString("name","http://www.webifysolutions.com/2005/10/catalog/governance#FabricProject");
		contextProperty_04.setString("value","fc://fabric/governance/fabricproject-inst#3c765894-632c-4866-810a-ac74e16b16b0");

		List propertyList = new ArrayList() ;
		//propertyList.add(contextProperty_01);
		propertyList.add(contextProperty_02);
		propertyList.add(contextProperty_03);
		propertyList.add(contextProperty_04);

		//context.setDataObject("context-identifier", contextId);
		context.setList("property",propertyList);
		LOG.info("context crearted");
		return context;
    }
	
	public void getSpecificDataObject(DataObject acord){
		java.util.List claimSvcRqBOList = acord.getList("ClaimsSvcRq");
		commonj.sdo.DataObject claimSvcRqBO = (commonj.sdo.DataObject)claimSvcRqBOList.get(0);
		java.util.List  ClaimStatusInqRqBOList = claimSvcRqBO.getList("ClaimStatusInqRq");
		commonj.sdo.DataObject ClaimStatusInqRqBO = (commonj.sdo.DataObject)ClaimStatusInqRqBOList.get(0);
		commonj.sdo.DataObject claimStatusInqInfoBO = ClaimStatusInqRqBO.getDataObject("ClaimStatusInqInfo");

		commonj.sdo.DataObject insuredOrPrincipalBO  = claimStatusInqInfoBO.getDataObject("InsuredOrPrincipal");

		commonj.sdo.DataObject generalPartyInfo = insuredOrPrincipalBO.getDataObject("GeneralPartyInfo");
		java.util.List  nameInfoBOList = generalPartyInfo.getList("NameInfo");
		commonj.sdo.DataObject nameInfoBO = (commonj.sdo.DataObject)nameInfoBOList.get(0);

		commonj.sdo.DataObject personNameBO = nameInfoBO.getDataObject("PersonName");

		commonj.sdo.DataObject surnameBO = personNameBO.getDataObject("Surname");

		commonj.sdo.DataObject givenNameBO = personNameBO.getDataObject("GivenName");

	}

}
