/***********************************************
 * Licensed Materials - Property of IBM
 * 
 * 5724-R31, 5655-S30
 *
 * (C) Copyright IBM Corp. 2002, 2006 All Rights Reserved.
 * (C) Copyright Webify Solutions, Inc. 2002, 2006 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ***********************************************
 */

package com.ibm.websphere.fabric.insurance.pc.service.util;

import java.math.BigDecimal;
import java.util.Date;

import com.ibm.websphere.fabric.da.util.SdoXPathUtil;
import com.ibm.websphere.fabric.da.util.XPathSearchResult;

import commonj.sdo.DataObject;

/**
 * Data assignments done in this util class.
 * 
 * @author govindm,harjinderm
 */
public final class DataAssigmentHelper {
	
    private DataAssigmentHelper() {
    }
    
    /**
     * Required details are set in the custDetailsResponse with the values from acordResponse.
     * 
     * @param acordResponse
     * @param custDetailsResponse
     */
    public static void assignCustomerDetailsToACORD(DataObject acordResponse, DataObject custDetailsResponse) {

        String firstname = (String) getValueFromXPath(custDetailsResponse, "//additionalNames[1]/firstName");        
        String lastname = (String) getValueFromXPath(custDetailsResponse, "//additionalNames[1]/lastName");

        String addr1 = ""; 
        String addr2 = ""; 
        String city = ""; 
        String state = ""; 
        String postcode = ""; 

        ClaimDetailInfo claimDetailInfo = (ClaimDetailInfo) DataReader.getInstance().getClaimDetailInfoByFirstName(firstname);

        /*if (firstname != null && !firstname.equals("")){
        	addr1 = (String) getValueFromXPath(custDetailsResponse, "//prospects[1]/contactPoint[1]/addressLine1");
            addr2 = (String) getValueFromXPath(custDetailsResponse, "//prospects[1]/contactPoint[1]/addressLine2");
            city = (String) getValueFromXPath(custDetailsResponse, "//prospects[1]/contactPoint[1]/city");
            state = (String) getValueFromXPath(custDetailsResponse, "//prospects[1]/contactPoint[1]/state");
            postcode = (String) getValueFromXPath(custDetailsResponse, "//prospects[1]/contactPoint[1]/postcode");
        }*/
        if (firstname != null && !firstname.equals("")){
        	addr1 = claimDetailInfo.getAddress1();
            addr2 = claimDetailInfo.getAddress2();
            city = claimDetailInfo.getCity();
            state = claimDetailInfo.getState();
            postcode = claimDetailInfo.getPostcode();
        }
        
        DataObject generalPartyInfoDO = acordResponse.createDataObject("ClaimsSvcRs")
            .createDataObject("ClaimStatusInqRs").createDataObject("ClaimStatusInqInfo")
            .createDataObject("InsuredOrPrincipal").createDataObject("GeneralPartyInfo");

        DataObject personNameDO = generalPartyInfoDO.createDataObject("NameInfo")
            .createDataObject("PersonName");
        personNameDO.createDataObject("GivenName").set("value", firstname);
        personNameDO.createDataObject("Surname").set("value", lastname);

        DataObject addrDO = generalPartyInfoDO.createDataObject("Addr");
        addrDO.createDataObject("Addr1").set("value", addr1);
        addrDO.createDataObject("Addr2").set("value", addr2);
        addrDO.createDataObject("City").set("value", city);
        addrDO.createDataObject("StateProvCd").set("value", state);
        addrDO.createDataObject("PostalCode").set("value", postcode);
    }

    /**
     * Required details are set in the retrieveInsurancePolicyDetailsResponse 
     * with the values from acordResponse.
     * 
     * @param acordResponse
     * @param retrieveInsurancePolicyDetailsResponse
     */
    public static void assignInsurancePolicyDetailsToACORD(DataObject acordResponse,
                                                           DataObject retrieveInsurancePolicyDetailsResponse) {
       
        String policyNo = (String) getValueFromXPath(retrieveInsurancePolicyDetailsResponse, 
            "//involvedParties[1]/partyRegistrations[1]/externalReference");

        DataObject claimStatusInqInfoDO = getDOFromXPath(acordResponse, 
            "//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimStatusInqInfo");
        
        claimStatusInqInfoDO.createDataObject("PolicyNumber").set("value", policyNo);
        
        
        //set lob in acord resp.
        
        
        
    }

    /**
     * Required details are set in the retrieveClaimDetailsResponse 
     * with the values from acordResponse.
     * 
     * @param acordResponse
     * @param retrieveClaimDetailsResponse
     */
    public static void assignClaimDetailsToACORD(DataObject acordResponse,
                                                 DataObject retrieveClaimDetailsResponse) {

        String accNoId = (String) getValueFromXPath(retrieveClaimDetailsResponse, 
            "//claimAccount/financialInstitution/financialInstitution/partyRegistrations[1]/externalReference");
        DataObject claimFolderDO = getDOFromXPath(retrieveClaimDetailsResponse, "//claimFolder");
        
        DataObject claimRelationshipsDO = getDOFromXPath(retrieveClaimDetailsResponse, "//claimRelationships");
        
        String claimStatusCd = claimFolderDO.getString("perilTypeExternalRef");
        Date claimStatusDt = claimFolderDO.getDate("closingDate");

        //notificationdate --->openingDate
        // Notification date >> acord->Reported Date >>openingDate
        Date reportedDt = claimFolderDO.getDate("openingDate");
        // Start date >> acord->Lossdate
        Date lossDt = claimRelationshipsDO.getDate("startDate");
        //String incidentDesc = claimFolderDO.getString("perspectiveOnClaimDescription");
        String incidentDesc = claimRelationshipsDO.getString("description");
        //String paymentTypeCd = claimFolderDO.getString("state");

        String paymentTypeCd = "VISA";
        	
        	
        DataObject claimStatusInqRsDO = getDOFromXPath(acordResponse, "//ClaimsSvcRs[1]/ClaimStatusInqRs[1]");
        
        DataObject claimStatusInqInfoDO = getDOFromXPath(claimStatusInqRsDO, "//ClaimStatusInqInfo");
        claimStatusInqInfoDO.createDataObject("AccountNumberId").set("value", accNoId);
                
        DataObject claimsSummaryInfoDO = claimStatusInqRsDO.createDataObject("ClaimsSummaryInfo");
        claimsSummaryInfoDO.createDataObject("ClaimStatusCd").set("value", claimStatusCd);

        if (claimStatusDt != null) {
            claimsSummaryInfoDO.createDataObject("ClaimStatusDt").set("value", claimStatusDt);
        }
        
        if (reportedDt != null) {
            claimsSummaryInfoDO.createDataObject("ReportedDt").set("value", reportedDt);
        }
        
        if (lossDt != null) {
            claimsSummaryInfoDO.createDataObject("LossDt").set("value", lossDt);
        }
        claimsSummaryInfoDO.createDataObject("IncidentDesc").set("value", incidentDesc);
        claimsSummaryInfoDO.createDataObject("ClaimsPayment").createDataObject("PaymentTypeCd").set("value", paymentTypeCd);
    }

    /**
     * Required details are set in the retrievePaymentDetailsResponse 
     * with the values from acordResponse.
     * 
     * @param acordResponse
     * @param retrievePaymentDetailsResponse
     */
    public static void assignPaymentDetailsToACORD(DataObject acordResponse,
                                                   DataObject retrievePaymentDetailsResponse) {
    	
	    BigDecimal totalPaymentAmt = new BigDecimal((String) getValueFromXPath(retrievePaymentDetailsResponse, 
	   	    "//relatedAccountEntries[1]/amount/theCurrencyAmount"));

	    DataObject relatedAccountEntriesDO = getDOFromXPath(retrievePaymentDetailsResponse, 
		        "//relatedAccountEntries[1]");
	    Date paymentDt = relatedAccountEntriesDO.getDate("postedDate");
	    String methodPaymentCd = (String) getValueFromXPath(retrievePaymentDetailsResponse, 
            "//payments/paymentMethod");

        DataObject claimsPaymentDO = getDOFromXPath(acordResponse, 
            "//ClaimsSvcRs[1]/ClaimStatusInqRs[1]/ClaimsSummaryInfo/ClaimsPayment[1]");
        claimsPaymentDO.createDataObject("TotalPaymentAmt").createDataObject("Amt")
            .set("value",totalPaymentAmt);
        if(methodPaymentCd != null && !methodPaymentCd.equals("")) {
            claimsPaymentDO.createDataObject("MethodPaymentCd").set("value", methodPaymentCd);
        }
        
        if(paymentDt != null) {
            claimsPaymentDO.createDataObject("PaymentDt").set("value", paymentDt);
        }
    }

    /**
     * Returns an object from the for DataObject with the given XPath.
     * 
     * @param objectDO
     * @param xpath
     * @return Object
     */
    public static Object getValueFromXPath(DataObject objectDO, String xpath) {
        Object value = null;
        XPathSearchResult xpsresult = SdoXPathUtil.getViaXpath(objectDO, xpath);
        if(xpsresult!= null) {
	        if (xpsresult.isDataObject()) {
                value = ((DataObject)xpsresult.getValue()).get("value");
            } else {
                value = xpsresult.getValue();
            }    	
	    }
        return value;
    }

    /**
     * Returns DataObject from the for DataObject with the given XPath.
     * 
     * @param objectDO
     * @param xpath
     * @return DataObject
     */
    public static DataObject getDOFromXPath(DataObject objectDO, String xpath) {
        XPathSearchResult xpsresult = SdoXPathUtil.getViaXpath(objectDO, xpath);
	    return (DataObject)xpsresult.getValue();
    }
}
